use std::{
    cmp::{PartialEq, PartialOrd}, ops::{Add, Sub}
};

use color_eyre::Result;
use crate::action::Action;

pub trait Scrollable {
    type IndexType: Add<Output = Self::IndexType>
        + Sub<Output = Self::IndexType>
        + PartialOrd
        + PartialEq
        + From<u8>
        + Copy;

    fn apply_next_row(&mut self, new_index: Self::IndexType) -> Result<Option<Action>>;
    fn apply_prev_row(&mut self, new_index: Self::IndexType) -> Result<Option<Action>>;

    fn selected_index(&self) -> Option<Self::IndexType>;
    fn items_length(&self) -> Self::IndexType; 

    fn next_row(&mut self) -> Result<Option<Action>> {
        let length = self.items_length();
        if length == 0.into() {
            return Ok(None);
        }

        let new_index = match self.selected_index() {
            Some(index) => {
                if index < length - 1.into() {
                    index + 1.into()
                } else {
                    index
                }
            },
            None => 0.into(),
        };
        self.apply_next_row(new_index)
    }

    fn prev_row(&mut self) -> Result<Option<Action>> {
        if self.items_length() == 0.into() {
            return Ok(None);
        }

        let new_index = match self.selected_index() {
            Some(index) => {
                if index == 0.into() {
                    0.into()
                } else {
                    index - 1.into()
                }
            }
            None => 0.into()
        };
        self.apply_prev_row(new_index)
    }
}
